/* Copyright (c) 2020 Kneron, Inc. All Rights Reserved.
 *
 * The information contained herein is property of Kneron, Inc.
 * Terms and conditions of usage are described in detail in Kneron
 * STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information.
 * NO WARRANTY of ANY KIND is provided. This heading must NOT be removed
 * from the file.
 */

/******************************************************************************
*  Filename:
*  ---------
*  main.c
*
*  Description:
*  ------------
*
*
******************************************************************************/

#include "cmsis_os2.h"
#include "project.h"
#include "kdrv_pinmux.h"
#include "system_init.h"
#include "driver_init.h"
#include "device_init.h"
#include "kmdw_console.h"
#include "kmdw_memory.h"
#include "kmdw_system.h"
#include "kmdw_tdc.h"
#include "kmdw_errandserv.h"
#include "kmdw_tdc.h"
#include "kmdw_model.h"
#include "version.h"

// inference app/core
#include "kp_struct.h"
#include "kmdw_inference_app.h"
#include "kdp2_inf_app_yolo.h"
#include "demo_customize_inf_single_model.h"
#include "demo_customize_inf_multiple_models.h"

// inference client
#include "kdp2_usb_companion.h"

#define MAX_IMAGE_COUNT 10
#define MAX_RESULT_COUNT 10

/* declare inference code implementation here */
extern void task_initialize(void);

int sdk_main(void)
{
    sys_initialize();
    drv_initialize();
    dev_initialize();
    osKernelInitialize();

    kmdw_ddr_init(DDR_HEAP_BEGIN, DDR_HEAP_END); // FIXME
    //kmdw_console_queue_init();                // uart console
    kmdw_console_set_log_level_scpu(LOG_DBG);

    //kmdw_tdc_start();

    kmdw_printf(">> Start running KL720 KDP2 companion mode ...\n");

    load_ncpu_fw(1);
    kmdw_model_init();
    //kmdw_errandserv_start(); // a service thread for running errand tasks (bottom-half)

//kapp_dsp_ipc_init(image_dfifo, result_dfifo);
/* Start RTOS Kernel */
#ifdef TDC_ENABLE
    kmdw_tdc_start();
#endif

    kmdw_printf("SDK v%u.%u.%u-build.%03u\n", (uint8_t)(IMG_FW_VERSION >> 24), (uint8_t)(IMG_FW_VERSION >> 16), (uint8_t)(IMG_FW_VERSION >> 8), (uint8_t)(IMG_FW_VERSION));

    return 0;
}

static void app_func(void *inf_input_buf)
{
    // check header stamp
    kp_inference_header_stamp_t *header_stamp = (kp_inference_header_stamp_t *)inf_input_buf;
    uint32_t job_id = header_stamp->job_id;

    switch (job_id)
    {
    case KDP2_INF_ID_APP_YOLO:
        kdp2_app_yolo_inference(job_id, inf_input_buf);
        break;
    case KDP2_JOB_ID_APP_YOLO_CONFIG_POST_PROC:
        kdp2_app_yolo_config_post_process_parameters(job_id, inf_input_buf);
        break;
    case DEMO_KL720_CUSTOMIZE_INF_SINGLE_MODEL_JOB_ID: // a demo code implementation in SCPU for user-defined/customized inference from one model
        demo_customize_inf_single_model(job_id, inf_input_buf);
        break;
    case DEMO_KL720_CUSTOMIZE_INF_MULTIPLE_MODEL_JOB_ID: // a demo code implementation in SCPU for user-defined/customized inference from multiple model
        demo_customize_inf_multiple_model(job_id, inf_input_buf);
        break;
    default:
        kmdw_inference_app_send_status_code(job_id, KP_FW_ERROR_UNKNOWN_APP);
        break;
    }
}

/******************************************************************************
Declaration of static Global Variables & Functions
******************************************************************************/
int main(void)
{
    /* SDK main init */
    sdk_main();

    /* initialize inference app */
    /* register APP functions */
    /* specify depth of inference queues */
    kmdw_inference_app_init(app_func, 10, 10);

    /* companion mode init */
    kdp2_usb_companion_init();

    task_initialize();

    /* Start RTOS Kernel */
    if (osKernelGetState() == osKernelReady)
    {
        osKernelStart();
    }

    while (1)
        ;
}
