#ifndef __KMDW_LOG_H__
#define __KMDW_LOG_H__

#include <stdio.h>
#include "kmdw_status.h"
#include "kdrv_uart.h"
#include "kmdw_memory.h"

#define LOG_NONE 0
#define LOG_CRITICAL 1
#define LOG_ERROR 2
#define LOG_USER 3

#define LOG_INFO 4
#define LOG_TRACE 5
#define LOG_DBG 6

#define LOG_PROFILE 9

typedef void (*print_callback)(const char *log);

extern kmdw_status_t kmdw_console_queue_init(void);
extern void kmdw_console_set_log_level_scpu(uint32_t level);
extern void kmdw_console_set_log_level_ncpu(uint32_t level);
extern char kmdw_console_getc(void);
extern void kmdw_console_putc(char Ch);
extern void kmdw_console_puts(char *str);
extern int kmdw_console_echo_gets(char *buf, int len);
extern void kmdw_console_hook_callback(print_callback print_cb);
extern uint32_t kmdw_console_get_log_level_scpu(void);
extern kmdw_status_t kmdw_printf(const char *fmt, ...);

extern kmdw_status_t kmdw_level_printf(int level, const char *fmt, ...);
kmdw_status_t kmdw_printf(const char *fmt, ...);
kmdw_status_t kmdw_level_printf(int level, const char *fmt, ...);
kmdw_status_t kmdw_level_ipc_printf(int level, const char *fmt, ...);

#ifdef LOG_ENABLE

#define DSG(__format__, ...)                         \
    {                                                \
        kmdw_printf(__format__ "\n", ##__VA_ARGS__); \
    }

#define DSG_NOLF(__format__, ...)               \
    {                                           \
        kmdw_printf(__format__, ##__VA_ARGS__); \
    }

#define dbg_msg(fmt, ...) kmdw_level_printf(LOG_DBG, "[DBG] "fmt, ##__VA_ARGS__)
#define trace_msg(fmt, ...) kmdw_level_printf(LOG_TRACE, "[TRACE] "fmt, ##__VA_ARGS__)
#define info_msg(fmt, ...) kmdw_level_printf(LOG_INFO, "[INFO] "fmt, ##__VA_ARGS__)
#define err_msg(fmt, ...) kmdw_level_printf(LOG_ERROR, "[ERROR] "fmt, ##__VA_ARGS__)
#define critical_msg(fmt, ...) kmdw_level_printf(LOG_CRITICAL, "[CRITICAL] "fmt, ##__VA_ARGS__)
#define profile_msg(fmt, ...) kmdw_level_printf(LOG_PROFILE, "[PROFILE] "fmt, ##__VA_ARGS__)
#define dlog(fmt, ...) kmdw_level_printf(LOG_DBG, "[%s][%s] " fmt "\r\n", DEF_LOG_CATEG, __func__, ##__VA_ARGS__)


#define ipc_dbg_msg(fmt, ...) kmdw_level_ipc_printf(LOG_DBG, "[DBG] "fmt, ##__VA_ARGS__)
#define ipc_trace_msg(fmt, ...) kmdw_level_ipc_printf(LOG_TRACE, "[TRACE] "fmt, ##__VA_ARGS__)
#define ipc_info_msg(fmt, ...) kmdw_level_ipc_printf(LOG_INFO, "[INFO] "fmt, ##__VA_ARGS__)
#define ipc_err_msg(fmt, ...) kmdw_level_ipc_printf(LOG_ERROR, "[ERROR] "fmt, ##__VA_ARGS__)
#define ipc_critical_msg(fmt, ...) kmdw_level_ipc_printf(LOG_CRITICAL, "[CRITICAL] "fmt, ##__VA_ARGS__)
#define ipc_profile_msg(fmt, ...) kmdw_level_ipc_printf(LOG_PROFILE, "[PROFILE] "fmt, ##__VA_ARGS__)
#define ipc_dlog(fmt, ...) kmdw_level_ipc_printf(LOG_DBG, "[%s][%s] " fmt "\r\n", DEF_LOG_CATEG, __func__, ##__VA_ARGS__)

#else //////////////////////////////////////////////////////////////////////

#define DSG_NOLF(__format__, ...)
#define DSG(__format__, ...)
#define dbg_msg(fmt, ...)
#define trace_msg(fmt, ...)
#define info_msg(fmt, ...)
#define err_msg(fmt, ...)
#define critical_msg(fmt, ...)
#define dlog(fmt, ...)

#endif // LOG_ENABLE ///////////////////////////////////////////////////////

#define dbg_msg_api(fmt, ...) dbg_msg(fmt, ##__VA_ARGS__)
#define dbg_msg_app(fmt, ...) dbg_msg(fmt, ##__VA_ARGS__)
#define dbg_msg_algo(fmt, ...) critical_msg(fmt, ##__VA_ARGS__)
#define dbg_msg_console(fmt, ...) critical_msg(fmt "\n", ##__VA_ARGS__)
#define dbg_msg_user(fmt, ...) kmdw_level_printf(LOG_USER, fmt, ##__VA_ARGS__)

#endif // __KMDW_LOG_H__
