/*
 * Kneron DDR memory manager
 *
 * Copyright (C) 2021 Kneron, Inc. All rights reserved.
 *
 */

#ifndef KMDW_DDR_MEM_MGR_H
#define KMDW_DDR_MEM_MGR_H

#include "com.h"

 /* application client ID, used a tag attached to every allocated DDR block
 */

typedef enum {
    FW_INFO_DDR,
    MODEL_DDR,
    CAM_IMG_DDR,
    FID_DDR,
    DB_DDR,
    SI_DDR,
    ISI_DDR,
    FD_AGE_GENDER_DDR,
    DME_DDR,
    JPEG_ENC_DDR,
    JPEG_DEC_DDR,
    HICO_DDR,
    TOTAL_DDR_CLIENTS,
} ddr_client_id_t;

/**
 * @brief to allocate DDR memory aligned (64 bytes alignment) for the specified client
 * @param id: application client ID
 * @param numbtye: size in byte
 * @return the address of allocated block
 */
uint32_t kmdw_ddr_malloc(ddr_client_id_t id, uint32_t numbyte);

/**
 * @brief free the specified DDR block for the specified client
 * @param id: application client ID
 * @param mem_addr: memory address
 */
void kmdw_ddr_free(ddr_client_id_t id, uint32_t mem_addr);

/**
 * @brief free all DDR blocks associated with the specified client
 * @param id: application client ID
 */
void kmdw_ddr_clean_client(ddr_client_id_t id);

/**
 * @brief inquery if the specified client has at least one associated DDR block
 * @param id: application client ID
 * @return TURE: has one assocated block at least; FALSE: none
 */
bool kmdw_ddr_is_mem_available(ddr_client_id_t id);

#endif

