/********************************************************************
 * Copyright (c) 2020 Kneron, Inc. All Rights Reserved.
 *
 * The information contained herein is property of Kneron, Inc.
 * Terms and conditions of usage are described in detail in Kneron
 * STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information.
 * NO WARRANTY of ANY KIND is provided. This heading must NOT be removed
 * from the file.
 ********************************************************************/

/**@addtogroup  KDRV_NPU KDRV_NPU
 * @{
 * @brief       Kneron NPU driver
 * @copyright   Copyright (C) 2020 Kneron, Inc. All rights reserved.
 */

#ifndef __KDRV_NPU_H_
#define __KDRV_NPU_H_

#include "cmsis_os2.h"
#include "ipc.h"
#if 0
typedef enum {
    CMD_ID_MEM_WRITE = 0x1,
    CMD_ID_MEM_READ = 0x2,
    CMD_ID_RUN_YOLO3 = 0x3,
    CMD_ID_STOP_YOLO_EXEC = 0x04,
    CMD_ID_NOTIFY_FW_INFO_LOADED = 0x05,
    CMD_ID_RUN_JPEG_ENC = 0x06,
    CMD_ID_RUN_JPEG_DEC = 0x07,
    CMD_ID_RUN_CROP_RESIZE = 0x08,
    CMD_ID_RUN_FD = 0x09,
    CMD_ID_RUN_LM,
    CMD_ID_RUN_FR,
    CMD_ID_RUN_CLASSIFY,
} presilicon_test_cmd_t;
#endif
typedef void (*ipc_handler_t)(struct kdp_img_raw_s *p_raw_image, int state);

/**
 * @brief Initialize NPU functionality
 * @param ipc_handler IPC callback
 */
void kmdw_ipc_initialize(ipc_handler_t ipc_handler);

/**
 * @brief Set model information
 * @param model_info_addr model information address
 * @param info_idx information index
 * @param slot_idx slot index
 */
void kmdw_ipc_set_model(struct kdp_model_s *model_info_addr, uint32_t info_idx, int32_t slot_idx);

/**
 * @brief Set active model index
 * @param index model slot index
 */
void kmdw_ipc_set_model_active(uint32_t index);

/**
 * @brief Set active image index
 * @param index image index
 */
void kmdw_ipc_set_image_active(uint32_t index);

/**
 * @brief Set SCPU debug level
 * @param lvl level
 */
void kdrv_ncpu_set_scpu_debug_lvl(uint32_t lvl);

/**
 * @brief Set NCPU debug level
 * @param lvl level
 */
void kdrv_ncpu_set_ncpu_debug_lvl(uint32_t lvl);

/**
 * @brief Trigger NCPU interrupt
 * @param ipc_idx IPC channel to trigger
 */
void kmdw_ipc_trigger_int(int ipc_cmd);

/**
 * @brief Get scpu_to_ncpu_t point
 * @return IPC struct
 */
scpu_to_ncpu_t* kmdw_ipc_get_output(void);

/**
 * @brief Get ncpu_to_scpu_result_t point
 * @return IPC struct
 */

ncpu_to_scpu_result_t* kmdw_ipc_get_input(void);


#endif
