/*
 * Kneron memory APIs
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */

#ifndef __KDP_MEMORY_H_
#define __KDP_MEMORY_H_

#include <stdint.h>

#define ALIGN_16BYTE    16
#define ALIGN_64BYTE    64

#define ALIGN16(n)          ((n + ALIGN_16BYTE - 1) & ~( ALIGN_16BYTE - 1))
#define ALIGN16_FLOOR(n)    ((n) & ~(ALIGN_16BYTE - 1))

#define ALIGN64(n)         ((n + ALIGN_64BYTE - 1) & ~(ALIGN_64BYTE - 1))
#define ALIGN64_FLOOR(n)   ((n) & ~(ALIGN_64BYTE - 1))

/**
 * @brief To initialize available DDR block
 * @param start_addr the start address of DDR block
 * @param eed_addr the end address of DDR block
 */
void kmdw_ddr_init(uint32_t start_addr, uint32_t end_addr);

/**
 * @brief to allocate DDR memory
 * @param numbtye size in byte
 * @return the address of allocated block
 */
uint32_t kmdw_ddr_reserve(uint32_t numbyte);

/**
 * @brief to allocate DDR memory aligned at 64 bytes
 * @param numbtye size in byte
 * @return the address of allocated block
 */
uint32_t kmdw_ddr_reserve64(uint32_t numbyte);

/**
 * @brief to free DDR memory
 * @param pMem - pointer to memory block
 */
void kmdw_ddr_release(uint32_t mem_addr);

#endif

