#ifndef __APP_H__
#define __APP_H__

#include <stdint.h>

/* These header defines structures shared by scpu/ncpu/host_lib */

#define LAND_MARK_POINTS       5
#define FR_FEAT_SIZE           256
#define LV_R_SIZE              1
#define DUAL_LAND_MARK_POINTS  10
#define DME_OBJECT_MAX         80
#define IMAGENET_TOP_MAX       5

#define FD_RES_LENGTH  (2 * 4)
#define FR_RES_LENGTH  (4 * FR_FEAT_SIZE)
#define LM_RES_LENGTH  (4 * LAND_MARK_POINTS)
#define LV_RES_LENGTH  4
#define SCORE_RES_LENGTH  4
#define MAX_CARS_DETECTED  20
#define MAX_CAR_PLATES     MAX_CARS_DETECTED
#define MAX_PLATE_STR_LEN  10
#define OCR_MAX_NUM 7

/* Yolo Result */
typedef struct bounding_box_s {
    float x1;      // top-left corner: x
    float y1;      // top-left corner: y
    float x2;      // bottom-right corner: x
    float y2;      // bottom-right corner: y
    float score;   // probability score
    int32_t class_num; // class # (of many) with highest probability
} bounding_box_t;

typedef struct yolo_result_s {
    uint32_t class_count;            // total class count
    uint32_t box_count;              // boxes of all classes
    struct bounding_box_s boxes[1];  // box_count
} yolo_result_t;

struct age_gender_result_s {
    uint32_t age;
    uint32_t ismale;
};

typedef struct imagenet_result_s {
    int32_t   index; // index of the class
    float score; // probability score of the class
}imagenet_result_t;

struct facedet_result_s {
    int32_t len;
    int32_t xywh[4]; // 4 values for X, Y, W, H
};

struct landmark_result_s {
    struct {
        uint32_t x;
        uint32_t y;
    } marks[LAND_MARK_POINTS];
    float score;
    float blur;
    int32_t class_num;
};

struct fr_result_s {
    float feature_map[FR_FEAT_SIZE];
};

struct lv_result_s{
    float real[LV_R_SIZE];
};

struct dual_landmarks_s {
    struct {
        uint32_t x;
        uint32_t y;
    } marks[DUAL_LAND_MARK_POINTS];
};

struct fd_landmark_result_s {
    struct bounding_box_s boxes;
    struct {
        uint32_t x;
        uint32_t y;
    } marks[LAND_MARK_POINTS];
    float score;
    float blur;
};

typedef struct {
    struct bounding_box_s fd_res;
    struct age_gender_result_s ag_res;
} fd_age_gender_res;

struct fd_age_gender_s {
    uint32_t count;
    fd_age_gender_res boxes[DME_OBJECT_MAX];
};

typedef struct {
    uint32_t class_count; // total class count
    uint32_t box_count;   // boxes of all classes
    struct bounding_box_s boxes[DME_OBJECT_MAX]; // box information
} dme_res;

typedef struct {
    uint32_t class_count; // total class count
    uint32_t box_count;   // boxes of all classes
    struct fd_landmark_result_s fd_lm_res[DME_OBJECT_MAX]; // box and landmark information
} fd_lm_res;


#define CENTERNET_DETECTION_SIZE 50
/* centernet result */
typedef struct {
    uint32_t    class_count;
    uint32_t    box_count;
    struct bounding_box_s boxes[CENTERNET_DETECTION_SIZE];
}cnet_res;

typedef struct {
    uint32_t start_offset;
    uint32_t buf_len;
    uint32_t node_id;
    uint32_t supernum;
    uint32_t data_format;
    uint32_t row_start;
    uint32_t col_start;
    uint32_t ch_start;
    uint32_t row_length;
    uint32_t col_length;
    uint32_t ch_length;
    uint32_t output_index;
    uint32_t output_radix;
    uint32_t output_scale;
} raw_onode_t;

typedef struct {
    uint32_t total_raw_len;
    int32_t   total_nodes;
    raw_onode_t onode_a[40];
    uint8_t data[];
}raw_cnn_res_t;

typedef struct {
    int16_t     vector1;
    int16_t     vector2;
    int16_t     vector3;
    int16_t     vector4;
} motion_vector_t;

/***** license plate structs *****/

/* car detect result */
typedef struct cd_result_s {
    uint32_t class_count;
    uint32_t box_count;  
    struct bounding_box_s boxes[MAX_CARS_DETECTED];
} cd_result_t;

/* license plate detect result */
typedef struct lpd_result_s {
    struct {
        uint32_t x;
        uint32_t y;
    } marks[LAND_MARK_POINTS];
}lpd_result_t;

/* license plate recog result */

typedef struct lpr_result_s
{
    uint32_t class_count;
    uint32_t chars_count;
    struct bounding_box_s char_boxs[MAX_PLATE_STR_LEN];
}lpr_result_t;

typedef struct ocr_result_s {
    /*copy from lpr_result_s to store each bounding box of ocr value*/
    uint32_t chars_count;
    struct bounding_box_s char_boxs[MAX_PLATE_STR_LEN];
    /*-------------------------------------------------------------*/
	uint8_t valid;                      // 0 for invalid, 1 for valid
    uint8_t value[OCR_MAX_NUM];         // integers representing characters in order, mapping
                                        //     defined in licenseplate_ocr.c
    uint8_t hyphen;                     // position after which the hyphen should occur
}ocr_result_t;
/***** end of car plate struct *****/

#endif
